/*===========================================================================
 * DESCRIPTION:
 *
 * This example exhibits basic functionality of the STM32F103 REva
 * daughterboard. It toggles the REva LEDs.
 *
 * JUMPERS:
 * On the REva board, you must set the following jumpers:
 *  OUTPUT: D0->D7
 *
 * Ensure that the BOOT Mode jumper on the daughterboard fits the project
 * settings (set to FLASH by default, but you can change it to RAM).
 *
 * Ensure that the two ICC jumpers are NOT connected, and that the "Not ICC"
 * jumpers are on.
 *
 *---------------------------------------------------------------------------
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 *---------------------------------------------------------------------------
 * Copyright (c) Raisonance S.A.S., 2007-2009
 *==========================================================================*/

/* Includes ------------------------------------------------------------------*/
#include "stm32f10x.h"
#include <string.h>
#include <stdio.h>

/* Private typedef -----------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
/* Private macro -------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/
USART_InitTypeDef USART_InitStructure;
TIM_TimeBaseInitTypeDef  TIM_TimeBaseStructure;
TIM_OCInitTypeDef  TIM_OCInitStructure;
__IO uint16_t CCR1_Val = 500;
//__IO int DoADC;

/* Private function prototypes -----------------------------------------------*/
void RCC_Configuration(void);
void NVIC_Configuration(void);
void GPIO_Configuration(void);
void TIM_Configuration(void);
void USART_Configuration(void);
void Delay(vu32 nCount);
void putsUSART(char Str[]);
void ADConverter_Init(void);
u16 Get_ADC1Value(void);

/* Private functions ---------------------------------------------------------*/
/*******************************************************************************
* Function Name  : main
* Description    : Main program.
* Input          : None
* Output         : None
* Return         : None
*******************************************************************************/
int main(void)
{
#ifdef DEBUG
  debug();
#endif

  int i, RxCnt, ADCCnt;
  unsigned int ADC_Value;
  unsigned int ADCBuf[1024];
  char Command[120];
  char Buffer[6];
  char sADC_Value[5] = ""; 

  // System Clocks Configuration
  RCC_Configuration();

  // NVIC Configuration
  NVIC_Configuration();

  // Configure the GPIO ports
  GPIO_Configuration();

  // Configure the Timer
//  TIM_Configuration();

  // Configure the 232C
  USART_Configuration();
  
  // Initialize the ADC
  ADConverter_Init();

  // ϐ̏
  RxCnt = 0;
//  DoADC = 0;  // for Timer

  while(1)
  {
    // USART1 Receive Data Register is not empty
    if (USART_GetFlagStatus(USART1, USART_FLAG_RXNE) == SET) {
      // Store the received byte in RxBuffer
      Command[RxCnt] = (USART_ReceiveData(USART1) & 0x7F);

			if (Command[RxCnt] == ')') {  // )̓R}h̏I	
				Command[RxCnt+1] = '\0';		// \0:NULL
			
				// Who	
				if (strncmp(Command, "Who", 3) == 0) {
        	putsUSART("OK:SIScope");
        }

      	// Version	
				else if (strncmp(Command, "Ver", 3) == 0) {
        	putsUSART("OK:Hard Ver=0.1, Firm Ver=0.1");
        }

 				// ADC	
				else if (strncmp(Command, "ADC", 3) == 0) {
					strcpy(Command, Command+4);	// ADC(폜
      		strncpy(Buffer, Command, 4);
					Buffer[4] = '\0';				// \0:NULL
					ADCCnt = atoi(Buffer);

          for (i=0; i<ADCCnt; i++) {        
            ADC_Value = Get_ADC1Value();
            ADC_Value &= 0xfff;
            ADCBuf[i] = ADC_Value;

            // TvOԊup
            // Pin PD.4=LED toggling
            GPIO_WriteBit(GPIOD, GPIO_Pin_4, (BitAction)(1 - GPIO_ReadOutputDataBit(GPIOD, GPIO_Pin_4)));
          }

          putsUSART("OK");
        }

       	// Read Data	
				else if (strncmp(Command, "ReadData", 8) == 0) {   
          for (i=0; i<ADCCnt-1; i++) {
            sprintf(sADC_Value,"%04d ", ADCBuf[i]);
            putsUSART(sADC_Value);
            putsUSART(",");
          }
          sprintf(sADC_Value,"%04d ", ADCBuf[ADCCnt-1]);
          putsUSART(sADC_Value);
        }

				// Command Error
				else {
					putsUSART("NG:");
          putsUSART(Command);
				}
	
				RxCnt = 0;
			}
			else {
				RxCnt++;
			}
    }
  }
}

// 񑗐M
void putsUSART(char Str[]) {
  int i;

  i = 0;
  while(Str[i] != '\0') {   // \0:NULL
    // Send one byte
    USART_SendData(USART1, Str[i]);
  
    // Loop until USART1 DR register is empty
    while(USART_GetFlagStatus(USART1, USART_FLAG_TXE) == RESET) {
    }
  
    i++;
  }
}

void ADConverter_Init(void)
{
    ADC_InitTypeDef  ADC_InitStructure;
    GPIO_InitTypeDef GPIO_InitStructure;
    
    // Enable PORTC clock 
    RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIOC, ENABLE);
    
    /* Configure pin as analog input ---------------*/
    GPIO_InitStructure.GPIO_Pin = GPIO_Pin_5;
    GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
    GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AIN;
    GPIO_Init(GPIOC, &GPIO_InitStructure);
    
    /* Enable ADC1 clock */
    RCC_APB2PeriphClockCmd( RCC_APB2Periph_ADC1, ENABLE );

    /* ADC1 Configuration ------------------------------------------------------*/
    ADC_InitStructure.ADC_Mode                = ADC_Mode_Independent;
    ADC_InitStructure.ADC_ScanConvMode        = DISABLE;
    ADC_InitStructure.ADC_ContinuousConvMode  = DISABLE;
    ADC_InitStructure.ADC_ExternalTrigConv    = ADC_ExternalTrigConv_None;
    ADC_InitStructure.ADC_DataAlign           = ADC_DataAlign_Right;
    ADC_InitStructure.ADC_NbrOfChannel        = 1; 
    ADC_Init( ADC1, &ADC_InitStructure );

    /* ADC1 Regular Channel 15 */
    ADC_RegularChannelConfig( ADC1, ADC_Channel_15, 1, ADC_SampleTime_1Cycles5);

    /* Enable ADC1 external trigger conversion */
    ADC_ExternalTrigConvCmd( ADC1, ENABLE );  

    /* Enable ADC1  */
    ADC_Cmd(ADC1, ENABLE);

    /* Enable ADC1 reset calibaration register */   
    ADC_ResetCalibration(ADC1);

    /* Check the end of ADC1 reset calibration register */
    while(ADC_GetResetCalibrationStatus(ADC1));

    /* Start ADC1 calibaration */
    ADC_StartCalibration(ADC1);

    /* Check the end of ADC1 calibration */
    while(ADC_GetCalibrationStatus(ADC1));   
}

u16 Get_ADC1Value()
{
    // Start ADC1 Software Conversion
    ADC_SoftwareStartConvCmd(ADC1, ENABLE);

     // Wait for end of conversion
    while (ADC_GetFlagStatus(ADC1, ADC_FLAG_EOC) == RESET);
        
    // Return the conversion value
    return (ADC_GetConversionValue(ADC1));
}

/*******************************************************************************
* Function Name  : RCC_Configuration
* Description    : Configures the different system clocks.
* Input          : None
* Output         : None
* Return         : None
*******************************************************************************/
void RCC_Configuration(void)
{
  /* RCC system reset(for debug purpose) */
  RCC_DeInit();

  /* Enable HSE */
  RCC_HSEConfig(RCC_HSE_ON);

  /* Wait till HSE is ready */
  while(RCC_GetFlagStatus(RCC_FLAG_HSERDY) == RESET)
  {;}

  /* Enable Prefetch Buffer */
  FLASH_PrefetchBufferCmd(FLASH_PrefetchBuffer_Enable);

  /* Flash 2 wait state */
  FLASH_SetLatency(FLASH_Latency_2);

  /* HCLK = SYSCLK */
  RCC_HCLKConfig(RCC_SYSCLK_Div1); 
  
  /* PCLK2 = HCLK */
  RCC_PCLK2Config(RCC_HCLK_Div1); 

  /* PCLK1 = HCLK/2 */
  RCC_PCLK1Config(RCC_HCLK_Div2);
  
  /* ADCCLK = PCLK2/6 */
  RCC_ADCCLKConfig(RCC_PCLK2_Div6); 

  /* PLLCLK = 12MHz * 6 = 72 MHz */
  RCC_PLLConfig(RCC_PLLSource_HSE_Div1, RCC_PLLMul_6);

  /* Enable PLL */ 
  RCC_PLLCmd(ENABLE);

  /* Wait till PLL is ready */
  while(RCC_GetFlagStatus(RCC_FLAG_PLLRDY) == RESET) {
  }

  /* Select PLL as system clock source */
  RCC_SYSCLKConfig(RCC_SYSCLKSource_PLLCLK);

  /* Wait till PLL is used as system clock source */
  while(RCC_GetSYSCLKSource() != 0x08) {
  }

  /* Enable USART1, GPIOA, GPIOD and AFIO clocks */
  RCC_APB2PeriphClockCmd(RCC_APB2Periph_USART1 | RCC_APB2Periph_GPIOA
      |RCC_APB2Periph_GPIOD | RCC_APB2Periph_AFIO, ENABLE);

  /* TIM2 clock enable */
  RCC_APB1PeriphClockCmd(RCC_APB1Periph_TIM2, ENABLE);
}

/*******************************************************************************
* Function Name  : GPIO_Configuration
* Description    : Configures the different GPIO ports.
* Input          : None
* Output         : None
* Return         : None
*******************************************************************************/
void GPIO_Configuration(void)
{
  GPIO_InitTypeDef GPIO_InitStructure;

	// PD.4=LED
	GPIO_InitStructure.GPIO_Pin = GPIO_Pin_4;
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_Out_PP;
	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
	GPIO_Init(GPIOD, &GPIO_InitStructure);	// ֐ǂݏo܂B

  /* Configure USART1 Tx (PA.09) as alternate function push-pull */
  GPIO_InitStructure.GPIO_Pin = GPIO_Pin_9;
  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF_PP;
  GPIO_Init(GPIOA, &GPIO_InitStructure);

  /* Configure USART1 Rx (PA.10) as input floating */
  GPIO_InitStructure.GPIO_Pin = GPIO_Pin_10;
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IN_FLOATING;
  GPIO_Init(GPIOA, &GPIO_InitStructure);
}

/* ---------------------------------------------------------------
  TIM2 Configuration: Output Compare Timing Mode:
  TIM2CLK = 72MHz, Prescaler = 7200, TIM2 counter clock = 10KHz
  CC1 update rate = TIM2 counter clock / CCR1_Val
--------------------------------------------------------------- */
void TIM_Configuration(void)
{
  /* Time base configuration */
  TIM_TimeBaseStructure.TIM_Period = 65535;
  TIM_TimeBaseStructure.TIM_Prescaler = 0;
  TIM_TimeBaseStructure.TIM_ClockDivision = 0;
  TIM_TimeBaseStructure.TIM_CounterMode = TIM_CounterMode_Up;

  TIM_TimeBaseInit(TIM2, &TIM_TimeBaseStructure);

  /* Prescaler configuration */
  TIM_PrescalerConfig(TIM2, 7200, TIM_PSCReloadMode_Immediate);

  /* Output Compare Timing Mode configuration: Channel1 */
  TIM_OCInitStructure.TIM_OCMode = TIM_OCMode_Timing;
  TIM_OCInitStructure.TIM_OutputState = TIM_OutputState_Enable;
  TIM_OCInitStructure.TIM_Pulse = CCR1_Val;
  TIM_OCInitStructure.TIM_OCPolarity = TIM_OCPolarity_High;

  TIM_OC1Init(TIM2, &TIM_OCInitStructure);

  TIM_OC1PreloadConfig(TIM2, TIM_OCPreload_Disable);

  /* TIM IT enable */
  TIM_ITConfig(TIM2, TIM_IT_CC1, ENABLE);

  /* TIM2 enable counter */
  TIM_Cmd(TIM2, ENABLE);
}

/* USART1 configuration ------------------------------------------
      - BaudRate = 460800 baud  
      - Word Length = 8 Bits
      - One Stop Bit
      - Parity No
      - Hardware flow control disabled (RTS and CTS signals)
      - Receive and transmit enabled
--------------------------------------------------------------- */
void USART_Configuration(void)
{
  USART_InitStructure.USART_BaudRate = 460800;
  USART_InitStructure.USART_WordLength = USART_WordLength_8b;
  USART_InitStructure.USART_StopBits = USART_StopBits_1;
  USART_InitStructure.USART_Parity = USART_Parity_No;
  USART_InitStructure.USART_HardwareFlowControl = USART_HardwareFlowControl_None;
  USART_InitStructure.USART_Mode = USART_Mode_Rx | USART_Mode_Tx;
  
  /* Configure USART1 */
  USART_Init(USART1, &USART_InitStructure);
  
  /* Enable the USART1 */
  USART_Cmd(USART1, ENABLE);
}

/*******************************************************************************
* Function Name  : NVIC_Configuration
* Description    : Configures Vector Table base location.
* Input          : None
* Output         : None
* Return         : None
*******************************************************************************/
void NVIC_Configuration(void)
{
#ifdef  VECT_TAB_RAM  
  /* Set the Vector Table base location at 0x20000000 */ 
  NVIC_SetVectorTable(NVIC_VectTab_RAM, 0x0); 
#else  /* VECT_TAB_FLASH  */
  /* Set the Vector Table base location at 0x08000000 */ 
  NVIC_SetVectorTable(NVIC_VectTab_FLASH, 0x0);   
#endif

  NVIC_InitTypeDef NVIC_InitStructure;

  /* Enable the TIM2 gloabal Interrupt */
  NVIC_InitStructure.NVIC_IRQChannel = TIM2_IRQn;
  NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = 0;
  NVIC_InitStructure.NVIC_IRQChannelSubPriority = 1;
  NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;

  NVIC_Init(&NVIC_InitStructure);
}

#ifdef  DEBUG
/*******************************************************************************
* Function Name  : assert_failed
* Description    : Reports the name of the source file and the source line number
*                  where the assert error has occurred.
* Input          : - file: pointer to the source file name
*                  - line: assert error line source number
* Output         : None
* Return         : None
*******************************************************************************/
void assert_failed(u8* file, u32 line)
{ 
  /* User can add his own implementation to report the file name and line number,
     ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */

  /* Infinite loop */
  while (1)
  {
  }
}
#endif

/*******************************************************************************
* Function Name  : Delay
* Description    : Inserts a delay time.
* Input          : nCount: specifies the delay time length.
* Output         : None
* Return         : None
*******************************************************************************/
void Delay(vu32 nCount)
{
  for(; nCount!= 0;nCount--);
}
